@@ -115,6 +115,7 @@ pub struct HttpHandler {
     pub hosted_directory: (String, PathBuf),
     pub follow_symlinks: bool,
     pub sandbox_symlinks: bool,
+    pub generate_listings: bool,
     pub check_indices: bool,
     pub strip_extensions: bool,
     /// (at all, log_colour)
@@ -153,6 +154,7 @@ impl HttpHandler {
             hosted_directory: opts.hosted_directory.clone(),
             follow_symlinks: opts.follow_symlinks,
             sandbox_symlinks: opts.sandbox_symlinks,
+            generate_listings: opts.generate_listings,
             check_indices: opts.check_indices,
             strip_extensions: opts.strip_extensions,
             log: (opts.loglevel < LogLevel::NoServeStatus, opts.log_colour),
@@ -719,6 +721,10 @@ impl HttpHandler {
             }
         }
 
+        if !self.generate_listings {
+            return self.handle_nonexistent(req, req_p);
+        }
+
         if client_mobile(&req.headers) {
             self.handle_get_mobile_dir_listing(req, req_p)
         } else {
@@ -1326,6 +1332,7 @@ impl Clone for HttpHandler {
             hosted_directory: self.hosted_directory.clone(),
             follow_symlinks: self.follow_symlinks,
             sandbox_symlinks: self.sandbox_symlinks,
+            generate_listings: self.generate_listings,
             check_indices: self.check_indices,
             strip_extensions: self.strip_extensions,
             log: self.log,
